import itertools
import sys

IS_PYTHON_2 = sys.version[0] == '2'

if IS_PYTHON_2:
    string_types = (str, unicode)
    zip_ = itertools.izip

    # noqa: E303

    def escape_string(string):
        return string.encode("string_escape")

else:  # python3 and above
    string_types = (str,)
    zip_ = zip

    # noqa: E303

    def escape_string(string):
        # in python3, 'string_escape' has become 'unicode_escape', but bytes is returned then,
        # that's why additional backward decode is needed
        return string.encode("unicode_escape").decode("utf-8")


def pairwise(iterable):
    if not iterable:
        return iterable
    a, b = itertools.tee(iterable)
    next(b, None)
    return zip_(a, b)



from examples.example2doc import get_clean_function_source


def some_decorator(*_):
    def wrapper(fcn):
        return fcn

    return wrapper


@some_decorator('''
documentation text
can span multiple
lines and contain sample_function text
''', '''digraph {
	graph [fontname=helvetica nodesep=0.5 ranksep=0.5 sep=0.4]
	n_e [label="It is\neaiser" shape=tab]
}''')
def sample_function(arg_1, *args):
    """
        some body of that function named sample_function
    """
    _ = (arg_1,) + args
    _ = """
        different text
    """
    # example ends here
    _ = """
        and another
    """


def test_get_clean_function_source_1():
    assert get_clean_function_source(sample_function) == '''\
def sample_function(arg_1, *args):
    _ = (arg_1,) + args
    _ = """
        different text
    """

'''


@some_decorator('', '')
def sample_function2():
    pass
    pass


def test_get_clean_function_source_2():
    assert get_clean_function_source(sample_function2) == '''\
@some_decorator('', '')
def sample_function2():
    pass
    pass

'''


@some_decorator('', '')
def sample_function3():
    pass
    # example ends here
    pass


def test_get_clean_function_source_3():
    assert get_clean_function_source(sample_function3) == '''\
@some_decorator('', '')
def sample_function3():
    pass

'''


def undecorated(things):
    """
        Some docstring
    """
    things += 2
    return things


def test_get_clean_function_source_4():
    assert get_clean_function_source(undecorated) == '''\
def undecorated(things):
    things += 2
    return things

'''


def undecorated_with_terminator(things):
    """
        Some docstring
    """
    things += 2
    # example ends here
    return things


def test_get_clean_function_source_5():
    assert get_clean_function_source(undecorated_with_terminator) == '''\
def undecorated_with_terminator(things):
    things += 2

'''



import hashlib


class IdVault(object):

    def __init__(self):
        self._known_ids = {}
        self._sub_graphs = set()

    def __call__(self, input_text, statement_arguments):
        long_hash = self._calc_hash(input_text, statement_arguments)
        already_known = self._find_in_known(long_hash)
        short_id = already_known or self._evaluate_shortest_unique_hash(long_hash)
        return already_known, short_id

    @staticmethod
    def _calc_hash(input_text, statement_arguments):
        attr_str = ", ".join([str(input_text)] + ["{}={}".format(k, v) for k, v in sorted(statement_arguments.items())])
        return hashlib.sha1(attr_str.encode("utf-8")).hexdigest()

    def _find_in_known(self, long_hash_string):
        for short, long in self._known_ids.items():
            if long == long_hash_string:
                return short

    def _evaluate_shortest_unique_hash(self, long_hash_string):
        for length in range(1, len(long_hash_string)):
            short_id = "n_" + long_hash_string[:length]
            if short_id not in self._known_ids:
                self._known_ids[short_id] = long_hash_string
                return short_id
            elif self._known_ids[short_id] == long_hash_string:
                return short_id
        raise ValueError("Id value collision: {}.".format(long_hash_string))

    def eval_sub_graph_name(self, dot_attrs):
        name = dot_attrs.pop("name", "cluster")
        if not name.startswith("cluster"):
            name = "cluster_" + name

        if name == "cluster" or name in self._sub_graphs:
            number = 0
            while True:
                new_name = "{}_{}".format(name, number)
                if new_name not in self._sub_graphs:
                    name = new_name
                    break
                number += 1

        self._sub_graphs.add(name)
        return name



import os
import shutil

import pytest

from examples.example2doc import append_ex_doc
from grot import string_types

THIS_DIR = os.path.dirname(__file__)
EXAMPLES_DIR = os.path.join(THIS_DIR, 'examples')
OUTPUT_README = os.path.join(EXAMPLES_DIR, "README.md")
VERBOSE_INFO = True


def info(text):
    if VERBOSE_INFO:
        print(text)


MARKDOWN_TEMPLATE = """\
{fdoc}
```python
{src}
```
Source of this example is in [{py_path}]({py_path}).

It generates raw dot-syntax text file in: [{dot_path}]({dot_path}).
And the final graph file is in: [{img_path}]({img_path}):

[![Rendered example image to be shown in gitlab)]({img_path}?raw=true "{fname}")]({img_path})

"""


def create_doc(example_fcn_name, examples_dir, fcn_docstring, py_relative_path, py_source_code, make_rel_path):
    base_out_file = os.path.join(examples_dir, 'out', example_fcn_name)
    dot_extension = ".gv"
    img_extension = ".png"
    graph_dot_path = base_out_file + dot_extension
    graph_image_path = base_out_file + dot_extension + img_extension
    output_graph = make_rel_path(graph_image_path)
    output_dot_path = make_rel_path(graph_dot_path)
    content = MARKDOWN_TEMPLATE.format(fname=example_fcn_name, fdoc=fcn_docstring,
                                       src=py_source_code,
                                       py_path=py_relative_path, img_path=output_graph, dot_path=output_dot_path)
    return content


def pytest_runtest_teardown(item, nextitem):
    assert isinstance(item, pytest.Function), "Unexpected things happened."
    if item.name.startswith("example"):
        print(item.config)
        append_ex_doc(item.function, create_doc, "examples/README.md")


@pytest.hookspec(firstresult=True)
def pytest_collect_directory(path, parent):
    if os.path.relpath(str(path), EXAMPLES_DIR) == '.':
        output_directory = os.path.join(EXAMPLES_DIR, 'out')

        to_remove = [p for p in (OUTPUT_README, output_directory) if os.path.exists(p)]
        if to_remove:
            info("\nRemoving example2doc outcomes:")
            for p in to_remove:
                info(" - {}".format(p))
                if os.path.isdir(p):
                    shutil.rmtree(p)
                else:
                    os.remove(p)
            info("")


def pytest_assertrepr_compare(op, left, right):
    if op == "==" and isinstance(left, string_types) and isinstance(right, string_types):
        if len(left) > 10 or len(right) > 10:
            compare_strings(left, right)


def side_by_side_diff(left, right):
    return """
--- (left):
{left}---
does not match currently defined reference:
--- (right):
{right}---

""".format(left=left, right=right)


def compare_strings(left, right):
    if left != right:
        print(side_by_side_diff(left, right))
    else:
        return True



import os


def example_01():
    """
        # Hello Grot
        To generate a graph you need to import Grot class, create its instance and
        define nodes and edges. While `g.edge()` call, you can pass unlimited
        number of nodes or plain strings (creates implicit node).

        If you don't connect given node (as `unconnected`) it's going to float somewhere around.
    """
    from grot import Grot

    this_dir_path = os.path.dirname(__file__)  # if run in console - remove 'directory' parameter below
    out_dir_path = os.path.join(this_dir_path, 'out')

    g = Grot(name='example_01', format='png', directory=out_dir_path, graph_attrs={"rankdir": "LR"})

    one = g.node("It is\neaiser")
    two = g.node("graphs", color="#8a9bac")
    ignored = g.node("Node floats when\nunconnected", color="#da3080")

    g.edge(one, "to define", two)
    g.render()

    # example ends here
    assert g.source == r"""digraph example_01 {
	graph [fontname=helvetica nodesep=0.5 rankdir=LR ranksep=0.5 sep=0.4]
	node [color="#13136c" fontname=helvetica penwidth=1.8 shape=box]
	edge [fontname=helvetica]
	n_a [label="It is\neaiser"]
	n_f [label=graphs color="#8a9bac"]
	n_3 [label="Node floats when\nunconnected" color="#da3080"]
	n_d [label="to define" shape=none]
	n_a -> n_d
	n_d -> n_f
}"""



import os


def example_02a():
    """
        # Branches

        One call of `g.edge(node1, node2, node3, ...)` creates single chain of arrows.
        To make a branch, you need to call `g.edge` once again.
    """
    from grot import Grot

    this_dir_path = os.path.dirname(__file__)  # if run in console - remove 'directory' parameter below
    out_dir_path = os.path.join(this_dir_path, 'out')
    g = Grot(name='example_02a', format='png', directory=out_dir_path, graph_attrs={"rankdir": "LR"})

    one = g.node("One")
    two = g.node("Two")
    three = g.node("Three")
    four = g.node("Four")
    five = g.node("Five")
    six = g.node("Six")
    seven = g.node("Seven")

    g.edge(one, two, three, four, 'A')
    g.edge(two, five, six, 'B')
    g.edge(two, seven, 'C')

    g.render()

    # example ends here
    assert g.source == r"""digraph example_02a {
	graph [fontname=helvetica nodesep=0.5 rankdir=LR ranksep=0.5 sep=0.4]
	node [color="#13136c" fontname=helvetica penwidth=1.8 shape=box]
	edge [fontname=helvetica]
	n_b [label=One]
	n_1 [label=Two]
	n_f [label=Three]
	n_c [label=Four]
	n_f8 [label=Five]
	n_b1 [label=Six]
	n_6 [label=Seven]
	n_3 [label=A shape=none]
	n_b -> n_1
	n_1 -> n_f
	n_f -> n_c
	n_c -> n_3
	n_d [label=B shape=none]
	n_1 -> n_f8
	n_f8 -> n_b1
	n_b1 -> n_d
	n_a [label=C shape=none]
	n_1 -> n_6
	n_6 -> n_a
}"""


def example_02b():
    """
    # Branches - syntax variant
    You don't have to assign nodes to variables. However it's a good practice to do so.
    You can define nodes while `g.edge(...)` call.
    Here node `two` is assigned to a local variable, because we refer it several times.
    """
    from grot import Grot

    this_dir_path = os.path.dirname(__file__)  # if run in console - remove 'directory' parameter below
    out_dir_path = os.path.join(this_dir_path, 'out')

    g = Grot(name='example_02b', format='png', directory=out_dir_path, graph_attrs={"rankdir": "LR"})

    two = g.node("Two")
    g.edge(g.node("One"), two, g.node("Three"), g.node("Four"), 'A')
    g.edge(two, g.node("Five"), g.node("Six"), 'B')
    g.edge(two, g.node("Seven"), 'C')

    g.render()

    # example ends here
    assert g.source == r"""digraph example_02b {
	graph [fontname=helvetica nodesep=0.5 rankdir=LR ranksep=0.5 sep=0.4]
	node [color="#13136c" fontname=helvetica penwidth=1.8 shape=box]
	edge [fontname=helvetica]
	n_1 [label=Two]
	n_b [label=One]
	n_f [label=Three]
	n_c [label=Four]
	n_3 [label=A shape=none]
	n_b -> n_1
	n_1 -> n_f
	n_f -> n_c
	n_c -> n_3
	n_f8 [label=Five]
	n_b1 [label=Six]
	n_d [label=B shape=none]
	n_1 -> n_f8
	n_f8 -> n_b1
	n_b1 -> n_d
	n_6 [label=Seven]
	n_a [label=C shape=none]
	n_1 -> n_6
	n_6 -> n_a
}"""



import inspect
import os
import re


def append_ex_doc(function, create_doc, output_doc_path="examples/README.md"):
    """
        It extracts docs and some info from function and then passes it to `create_doc` function
        that is supposed to be responsible for building a documentation.
        Then the result is appended to `output_doc_path`.

        The reason to handle it in this way is to avoid lies in example / documentation.
        The documentation is build out of test or example function's outcome.
        If test/example breaks, doc creation breaks as well.

        Doctest tools I know are way too primitive. I want it other way around. First python code - then doc.
    """

    example_fcn_name = function.__name__
    py_source_code = get_clean_function_source(function)
    doc_text = inspect.getdoc(function) or "# {}\n".format(example_fcn_name)

    def make_rel_path(path_):
        """Function to create links relative to output doc file."""
        return os.path.relpath(path_, os.path.dirname(output_doc_path))

    py_source_file = inspect.getsourcefile(function)
    examples_dir = os.path.dirname(py_source_file)
    py_relative_path = make_rel_path(py_source_file)

    content = create_doc(example_fcn_name, examples_dir, doc_text, py_relative_path, py_source_code, make_rel_path)

    with open(output_doc_path, 'at') as f:
        f.write(content)


def get_clean_function_source(function):
    """
        'clean' means without a decorator and without function docstring.

        `inspect.getsource` returns the function body with decorator and with it's doc-string.
        We would rather to get just source code.
        This one uses a parametrized regex to substitute decorator, docstring and everything
        starting from a terminator `# example ends here`
    """
    fcn_name = function.__name__
    find_pattern = r".*def {}(.+?)\n\s*(?:'''.*?'''|\"\"\".*?\"\"\"|)\s*(\n.*)".format(fcn_name)
    sub_pattern = r"def {}\1\2".format(fcn_name)
    terminator_pattern = r"(?:\s*#\s*example ends here.*)"
    whole_body = inspect.getsource(function)
    without_docstring = re.sub(find_pattern, sub_pattern, whole_body, 0, re.MULTILINE | re.DOTALL)
    return re.sub(terminator_pattern, "\n", without_docstring, 1, re.MULTILINE | re.DOTALL) + "\n"



import pytest

from grot import Grot


@pytest.fixture
def default_graph():
    class PG(Grot):
        pass

    return PG


@pytest.fixture
def plain_graph_class():
    class Plain(Grot):
        _default_graph_attrs = {}
        _default_node_attrs = {}
        _default_edge_attrs = {}

    return Plain


@pytest.fixture
def gph(plain_graph_class):
    return plain_graph_class(__file__)


def test_plain_with_attributes(plain_graph_class):
    g = plain_graph_class(__file__, graph_attrs={'color': '#caffee'}, edge_attrs={'penwidth': '1.1'})
    assert g.source == r"""digraph test_grot {
	graph [color="#caffee"]
	edge [penwidth=1.1]
}"""


def test_default_attribute_canceling(default_graph):
    g = default_graph(
        __file__,
        graph_attrs={'color': '#caffee'},
        node_attrs={'color': None, 'shape': None},
        edge_attrs={'fontname': None, 'penwidth': '1.1'}
    )
    assert g.source == r"""digraph test_grot {
	graph [color="#caffee" fontname=helvetica nodesep=0.5 ranksep=0.5 sep=0.4]
	node [fontname=helvetica penwidth=1.8]
	edge [penwidth=1.1]
}"""


def test_basic(default_graph):
    g = default_graph(__file__)
    phy_br = g.node("node\nwith\nlinebreaks", shape="note")
    phy_br.edge("a target")
    target_2 = g.node("explicit target", shape="box3d")
    phy_br.edge(target_2, "plain\nnode", color="#123456")

    assert g.source == r"""digraph test_grot {
	graph [fontname=helvetica nodesep=0.5 ranksep=0.5 sep=0.4]
	node [color="#13136c" fontname=helvetica penwidth=1.8 shape=box]
	edge [fontname=helvetica]
	n_5 [label="node\nwith\nlinebreaks" shape=note]
	n_a [label="a target" shape=none]
	n_5 -> n_a
	n_0 [label="explicit target" shape=box3d]
	n_4 [label="plain\nnode" shape=none]
	n_5 -> n_0 [color="#123456"]
	n_0 -> n_4 [color="#123456"]
}"""


def test_empty_subgraph(gph):
    with gph.subgraph(name="thing"):
        pass

    assert gph.source == """\
digraph test_grot {
	subgraph cluster_thing {
	}
}"""


def test_subgraph_node(gph):
    with gph.subgraph() as c0:
        c0.node('one')
        c0.node('two')
    gph.node('three')

    assert gph.source == """\
digraph test_grot {
	subgraph cluster_0 {
		n_f [label=one]
		n_a [label=two]
	}
	n_b [label=three]
}"""


def test_subgraph_edge_inside_host(gph):
    with gph.subgraph() as c0:
        one = c0.node('one')
        two = c0.node('two')
        one.edge(two)

    assert str(gph) == """\
digraph test_grot {
	subgraph cluster_0 {
		n_f [label=one]
		n_a [label=two]
		n_f -> n_a
	}
}"""


def test_subgraph_aux_edge(gph):
    with gph.subgraph() as c0:
        one = c0.node('one')
        two = c0.node('two', shape="box")
    gph.edge(one, two, 'three', 'four', penwidth='1.2')

    assert gph.source == """\
digraph test_grot {
	subgraph cluster_0 {
		n_f [label=one]
		n_e [label=two shape=box]
	}
	n_c [label=three shape=none]
	n_b [label=four shape=none]
	n_f -> n_e [penwidth=1.2]
	n_e -> n_c [penwidth=1.2]
	n_c -> n_b [penwidth=1.2]
}"""


@pytest.mark.parametrize("option", ['node', 'edge', 'subgraph'])
def test_subgraph_forbidden_making_edges(gph, option):
    with gph.subgraph() as c0:
        one = c0.node('one')

    message = r"This graph object named 'cluster_0' \('Plain' type\) has been finalized already."
    with pytest.raises(ValueError, match=message):
        if option == 'node':
            c0.node('one')
        elif option == 'edge':
            one.edge('two')
        else:
            with c0.subgraph():
                pass


@pytest.mark.parametrize("option", ["in", "aux"])
def test_edges_with_labels(option, gph):
    one = gph.node("one")
    two = gph.node("two")
    five = gph.node("five")

    if option == "aux":
        one.edge(("1-2", two), "three", ('2-3', "four"), five)
    else:
        gph.edge(one, ("1-2", two), "three", ('2-3', "four"), five)

    expected_result = r"""digraph test_grot {
	n_f [label=one]
	n_a [label=two]
	n_4 [label=five]
	n_c [label=three shape=none]
	n_b [label=four shape=none]
	n_f -> n_a [label="1-2"]
	n_a -> n_c
	n_c -> n_b [label="2-3"]
	n_b -> n_4
}"""

    assert str(gph) == expected_result


def test_first_node_in_edge_cannot_have_label(gph):
    with pytest.raises(ValueError, match="First edge in a chain cannot have a transition label."):
        gph.edge(("forbidden label", 'one'), 'two')



import os
from contextlib import contextmanager

import graphviz

from ._id_vault import IdVault
from ._tools import escape_string, string_types, pairwise


class Grot(object):
    graph_attributes = {}
    _default_graph_attrs = {
        'fontname': 'helvetica',
        'nodesep': '0.5',
        'ranksep': '0.5',
        'sep': '0.4',
    }
    node_attributes = {}
    _default_node_attrs = {
        'color': '#13136c',
        'shape': 'box',
        'fontname': 'helvetica',
        'penwidth': '1.8',
    }
    edge_attributes = {}
    _default_edge_attrs = {
        'fontname': 'helvetica',
    }

    def __init__(self, py_file_path=None, graph_attrs=None, node_attrs=None, edge_attrs=None, id_vault=None,
                 directed=True, **native_dot_kwargs):
        # the vault has to be shared between parent and child graphs
        self._id_vault = id_vault or IdVault()

        if isinstance(py_file_path, str) and os.path.exists(os.path.dirname(py_file_path)):
            auto_directory, py_file_name = os.path.split(py_file_path)
            auto_name = os.path.splitext(py_file_name)[0]

            native_dot_kwargs['directory'] = native_dot_kwargs.get('directory', auto_directory)
            native_dot_kwargs['name'] = native_dot_kwargs.get('name', auto_name)
            native_dot_kwargs['filename'] = native_dot_kwargs.get('filename', auto_name + ".dot")

        if directed:
            self.g = graphviz.Digraph(**native_dot_kwargs)
        else:
            self.g = graphviz.Graph(**native_dot_kwargs)

        attributes_sections = [
            ("graph", graph_attrs or {}, self.graph_attributes, self._default_graph_attrs),
            ("node", node_attrs or {}, self.node_attributes, self._default_node_attrs),
            ("edge", edge_attrs or {}, self.edge_attributes, self._default_edge_attrs),
        ]

        for section_name, instance_attributes, class_attributes, default_attributes in attributes_sections:
            used_attributes = default_attributes.copy()
            used_attributes.update(class_attributes)
            used_attributes.update(instance_attributes)
            # setting an attribute to None cancels default value
            used_attributes = {k: v for k, v in used_attributes.items() if v is not None}
            if used_attributes:
                self.g.attr(section_name, **used_attributes)

    def attr(self, *p, **k):
        return self.g.attr(*p, **k)

    @property
    def source(self):
        return self.g.source

    def render(self, *p, **k):
        if 'view' not in k:
            k['view'] = False
        return self.g.render(*p, **k)

    def same_rank(self, first, second, *others):
        self.g.body.append("{{rank=same; {}; {};{}}}".format(first, second, "".join(" %s;" % n for n in others)))

    def __str__(self):
        return str(self.g)

    @contextmanager
    def subgraph(self, graph_attrs=None, node_attrs=None, edge_attrs=None, label=None, **dot_attrs):
        graph_attrs = graph_attrs or {}
        if label:
            graph_attrs['label'] = label

        name = self._id_vault.eval_sub_graph_name(dot_attrs)
        sub_graph = self.__class__(name=name, graph_attrs=graph_attrs, node_attrs=node_attrs, edge_attrs=edge_attrs,
                                   id_vault=self._id_vault, **dot_attrs)

        yield sub_graph
        self.g.subgraph(graph=sub_graph.g)
        # after this moment this sub_graph instance becomes useless.
        # Creation of any object via this instance will be not attached to parent graph / will be not rendered.
        # In order to avoid enormous confusion - it's better to raise instantly:
        sub_graph.edge = sub_graph._raise_finalized
        sub_graph.node = sub_graph._raise_finalized
        sub_graph.subgraph = sub_graph._raise_finalized

    def _raise_finalized(self, *_, **__):
        msg = "This graph object named '{}' ('{}' type) has been finalized already.\n" \
              "You have to either use it within its 'with' subgraph scope\n" \
              "or use main graph's handle to create objects."
        raise ValueError(msg.format(self.g.name, self.__class__.__name__))

    def edge(self, start_node, target_node, *next_targets, **edge_attributes):
        edge_chain = self._chain(start_node, target_node, *next_targets)
        nodes = list(self._supplement_missing_nodes(edge_chain))
        if nodes and nodes[0][0] is not None:
            raise ValueError("First edge in a chain cannot have a transition label.")
        for (_, a), (edge_label, b) in pairwise(nodes):
            if edge_label is not None:
                current_edge_attrs = edge_attributes.copy()
                current_edge_attrs['label'] = escape_string(edge_label)
            else:
                current_edge_attrs = edge_attributes

            self.g.edge(str(a), str(b), **current_edge_attrs)

    def __getitem__(self, keys):
        if not isinstance(keys, tuple):
            keys = (keys,)
        if len(keys) < 2:
            raise ValueError("Expecting at least two points to make an edge.")

        return self.edge(*keys)

    def node(self, content_text, **attrs):
        yet_known, short_identifier = self._id_vault(content_text, attrs)
        if not yet_known:
            # register a new node in Dot instance
            self.g.node(short_identifier, label=escape_string(content_text), **attrs)

        return Node(self, short_identifier)

    @staticmethod
    def _chain(*elements):
        return elements

    def _supplement_missing_nodes(self, nodes_chain):
        for node_or_tuple in nodes_chain:
            if isinstance(node_or_tuple, tuple):
                if not len(node_or_tuple) == 2:
                    msg = "Expecting pairs, got {} elements in {}."
                    raise TypeError(msg.format(len(node_or_tuple), type(node_or_tuple).__name__))

                edge_label, node = node_or_tuple
            else:
                node = node_or_tuple
                edge_label = None

            if edge_label and not isinstance(edge_label, string_types):
                msg = "Expecting edge label given in string, got '{}' in '{}' node."
                raise TypeError(msg.format(type(edge_label).__name__, node))

            if not isinstance(node, Node):
                assert isinstance(node, string_types), "expecting node object or string, got " + str(type(node))
                yield edge_label, self.node(node, shape="none")
            else:
                yield edge_label, node


class Node(object):
    def __init__(self, parent_graph, unique_id):
        self.parent_graph = parent_graph
        self.unique_id = unique_id

    def __str__(self):
        return self.unique_id

    def edge(self, target_node, *next_targets, **edge_attributes):
        self.parent_graph.edge(self, target_node, *next_targets, **edge_attributes)



# -*- coding: utf8
from setuptools import setup

with open("README.rst") as f:
    long_description = f.read()

with open('requirements.txt') as f:
    lines = [line.strip() for line in f.readlines() if line]
    required = [line for line in lines if not line.startswith('#') and not line.startswith('-i')]

setup(
    name='grot',
    version='0.1.1',
    author='Michal Kaczmarczyk',
    author_email='michal.s.kaczmarczyk@gmail.com',
    maintainer='Michal Kaczmarczyk',
    maintainer_email='michal.s.kaczmarczyk@gmail.com',
    license='MIT license',
    url='https://gitlab.com/kamichal/grot',
    description='Graphviz syntax wrapper. Draw graphs with pure python.',
    long_description=long_description,
    long_description_content_type='text/x-rst',
    packages=['grot'],
    install_requires=required,
    keywords='',
    classifiers=[
        # https://pypi.org/pypi?%3Aaction=list_classifiers
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Programming Language :: Python',
        'Topic :: Documentation',
        'Topic :: Multimedia :: Graphics :: Presentation',
        'Topic :: Multimedia :: Graphics',
        'Topic :: Utilities',
    ],
)



from grot import IdVault


def test_id_vault():
    vault = IdVault()

    # check that when asked first time - calculates unique ids for different call statements
    assert [(k,) + vault("content", {'arg': k}) for k in range(4)] == [
        (0, None, 'n_7'),
        (1, None, 'n_f'),
        (2, None, 'n_d'),
        (3, None, 'n_79'),
    ]
    # check that when asked second time - indicates it's already known hash
    assert [(k,) + vault("content", {'arg': k}) for k in range(6)] == [
        (0, 'n_7', 'n_7'),
        (1, 'n_f', 'n_f'),
        (2, 'n_d', 'n_d'),
        (3, 'n_79', 'n_79'),
        (4, None, 'n_9'),
        (5, None, 'n_90')
    ]

    assert vault._known_ids == {
        'n_7': '7dd1d46df95a4069950b04f0f088f0a0fe787ced',
        'n_79': '79f0cadf37c16c170065d7b9661f24eb33f16e56',
        'n_9': '9c6e1ada077d2eb7f702646456be76b88d78fa96',
        'n_90': '9092f9b2bbf7d9468efe6e57599537dbdefe506f',
        'n_d': 'd418a2bc11706f0bbc4b57a7a5a90ea7f2e6d67a',
        'n_f': 'f9f31b0d213f2fce1e478783a2bb16d59dbc6cb6',
    }


def test_subgraph_names():
    vault = IdVault()

    names = [vault.eval_sub_graph_name({}) for _ in range(4)]
    assert names == ['cluster_0', 'cluster_1', 'cluster_2', 'cluster_3']
    assert vault.eval_sub_graph_name({'name': 'bad_name'}) == "cluster_bad_name"
    assert vault.eval_sub_graph_name({'name': 'bad_name'}) == "cluster_bad_name_0"
    assert vault.eval_sub_graph_name({'name': 'cluster_bad_name'}) == "cluster_bad_name_1"
    assert vault.eval_sub_graph_name({'name': 'cluster_bad_name'}) == "cluster_bad_name_2"
    assert vault.eval_sub_graph_name({'name': 'bad_name'}) == "cluster_bad_name_3"
